import os
from wazuh_evidence_collector.wazuh_client import WazuhClient
from elasticsearch import Elasticsearch
from forward_evidence.forward_evidence import ForwardEvidence
from forward_evidence.generate_evidence import create_resource, create_assessevidence_request, print_evidence
from forward_evidence.clouditor_authentication import ClouditorAuthentication
from wazuh_evidence_collector.checker import Checker
from wazuh_evidence_collector.demo_checker import DemoChecker
import uuid
import configparser
import logging.config

logging.config.fileConfig('logging.conf')
LOGGER = logging.getLogger('root')

DEMO = os.environ.get("demo_mode").lower() in ('true', '1', 't')

WAZUH_HOST = os.environ.get("wazuh_host")
WAZUH_PORT = int(os.environ.get("wazuh_port"))
WAZUH_USERNAME = os.environ.get("wazuh_username")
WAZUH_PASSWORD = os.environ.get("wazuh_password")

ELASTIC_HOST = os.environ.get("elastic_host")
ELASTIC_PORT = int(os.environ.get("elastic_port"))
ELASTIC_USERNAME = os.environ.get("elastic_username")
ELASTIC_PASSWORD = os.environ.get("elastic_password")

if not DEMO:
    wc = WazuhClient(WAZUH_HOST, WAZUH_PORT, WAZUH_USERNAME, WAZUH_PASSWORD, LOGGER)

    es = Elasticsearch(
            ELASTIC_HOST,
            http_auth=(ELASTIC_USERNAME, ELASTIC_PASSWORD),
            scheme='https',
            port=ELASTIC_PORT,
            use_ssl=False,
            verify_certs=False,
            ssl_show_warn=False,
        )

oauth_client = ClouditorAuthentication(LOGGER)

forwarder = ForwardEvidence(LOGGER)

# Get ID (UUID)
def get_id():
    id = uuid.uuid1()

    return id

# Get tool ID (SERVICE:VERSION format)
def get_tool_id():
    with open('MANIFEST', 'r') as f:
        config_string = '[clouditor]\n' + f.read()

    config = configparser.ConfigParser()
    config.read_string(config_string)

    version = '{}:{}'.format(config.get('clouditor', 'SERVICE'), config.get('clouditor', 'VERSION'))

    return version

def main():
    try:
        run_collector()
    except BaseException as e:
        LOGGER.exception("Exception caught in run_collector()")

# Wrapper function that runs all the checks (for every manager/agent)
def run_collector():
    checker = Checker(wc, es, LOGGER) if not DEMO else DemoChecker()

    # Get list of all agent ids (including manager's)
    def get_agents(wc):
        body = wc.req('GET', 'agents')

        agent_list = []
        for agent in body['data']['affected_items']:
            agent_list.append([agent['id'], agent['name']])

        return body, agent_list

    body, agent_list = get_agents(wc) if not DEMO else ({}, [[0, "dummyAgent0"], [1, "dummyAgent1"]])

    ae_req_list = []

    for agent in agent_list:
        ae_req_list.append(generate_evidence(agent, checker))

    # TODO:
    for ae_req in ae_req_list:
        forwarder.send_evidence(ae_req, oauth_client.get_token())
        print_evidence(LOGGER, ae_req.evidence)

    return ae_req_list

# Run checks and generate evidence
def generate_evidence(agent, checker):
    raw_evidence = []

    # MalwareProtectionEnabled
    evidence, result_syscheck = checker.check_syscheck(agent)
    raw_evidence.append(evidence)

    evidence, result_rootcheck = checker.check_rootcheck(agent)
    raw_evidence.append(evidence)

    evidence, result_virus_total = checker.check_virus_total_integration()
    raw_evidence.append(evidence)

    evidence, result_clamd_process = checker.check_clamd_process(agent)
    raw_evidence.append(evidence)

    evidence, result_clamd_logs = checker.check_clamd_logs_elastic(agent)
    raw_evidence.append(evidence)

    security_events_count = checker.check_security_events(agent)

    if result_syscheck and result_rootcheck and \
        (result_virus_total or (result_clamd_process and result_clamd_logs)):
        malware_protection = { "malwareProtection": { "enabled": True }}
    else: 
        malware_protection = { "malwareProtection": { "enabled": False }}

    # TODO: implement metrics
    malware_protection["malwareProtection"].update({ "daysSinceActive": None, "numberOfThreatsFound": security_events_count})

    # MalwareProtectionOutput
    evidence, result_alert_integration = checker.check_alert_integrations()
    raw_evidence.append(evidence)

    if result_alert_integration:
        malware_protection["malwareProtection"].update({ "applicationLogging": { "enabled": True, "loggingService": ["?"], "retentionPeriod": None }})
    else: 
        malware_protection["malwareProtection"].update({ "applicationLogging": { "enabled": False, "loggingService": [], "retentionPeriod": None }})

    resource = create_resource(agent[1], None, malware_protection)
    return create_assessevidence_request(get_id(), "evidence_collector_service", get_tool_id(), raw_evidence, resource)

if __name__ == "__main__":
    main()